import os
import requests
import time
import random 
import re
from pathlib import Path
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

def fetch_baidu_image_url(keyword):
    """获取百度图片的URL"""
    options = webdriver.ChromeOptions()
    options.add_argument("--headless")
    options.add_argument("--disable-gpu")
    options.add_argument("--no-sandbox")
    options.add_argument("--disable-dev-shm-usage")
    
    try:
        # 确保使用正确的 ChromeDriver 安装方式
        driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=options)
        search_url = f"https://image.baidu.com/search/index?tn=baiduimage&word={keyword}"
        driver.get(search_url)
        
        # 显式等待图片加载
        WebDriverWait(driver, 10).until(
            lambda d: d.find_element(By.CSS_SELECTOR, "img.main_img")
        )
        
        # 获取前10张图片
        images = driver.find_elements(By.CSS_SELECTOR, "img.main_img")[:10]
        
        if not images:
            return None
        
        # 随机选择一张图片
        random_img = random.choice(images)
        img_url = random_img.get_attribute("src")
        
        # 如果返回的是 base64 缩略图，则尝试获取真实 URL
        if img_url and img_url.startswith("data:image"):
            img_url = random_img.get_attribute("data-imgurl")
        
        return img_url if img_url else None
    
    except Exception as e:
        print(f"图片获取失败: {str(e)}")
        return None
    finally:
        if 'driver' in locals():
            driver.quit()

def generate_article(api_key, prompt, max_tokens=1500, temperature=1.3):
    """调用DeepSeek API生成文章"""
    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json"
    }

    payload = {
        "model": "deepseek-chat",
        "messages": [
            {"role": "user", "content": prompt}
        ],
        "temperature": temperature,
        "max_tokens": max_tokens
    }

    try:
        response = requests.post(
            "https://api.deepseek.com/v1/chat/completions",
            headers=headers,
            json=payload,
            proxies={"http": None, "https": None},
            timeout=60
        )
        response.raise_for_status()
        return response.json()['choices'][0]['message']['content']
    except Exception as e:
        print(f"API请求失败: {str(e)}")
        return None

def insert_image_to_article(content, image_url, topic):
    """在文章第二章节前插入图片"""
    if not image_url:
        return content
    
    # 查找第二个<h3>标签的位置
    first_h3 = content.find("</h3>")
    if first_h3 == -1:
        return content
    
    second_h3_pos = content.find("<h3>", first_h3)
    if second_h3_pos == -1:
        return content
    
    # 插入图片HTML代码
    img_html = f'\n<div style="text-align:center;margin:20px 0">\n<img src="{image_url}" alt="{topic}" style="max-width:100%">\n</div>\n'
    return content[:second_h3_pos] + img_html + content[second_h3_pos:]

def extract_h1_title(content):
    """从文章中提取h1标题"""
    h1_start = content.find("<h1>")
    if h1_start == -1:
        return None
    h1_end = content.find("</h1>", h1_start)
    if h1_end == -1:
        return None
    title = content[h1_start+4:h1_end].strip()
    # 清理标题中的特殊符号
    title = "".join([c if c.isalnum() or c in ('?', '？') else '' for c in title])
    return title

def save_to_txt(content, topic, save_dir):
    """保存内容到txt文件，使用h1标题作为文件名"""
    try:
        Path(save_dir).mkdir(parents=True, exist_ok=True)
        
        # 提取h1标题
        h1_title = extract_h1_title(content)
        
        if h1_title:
            filename = f"{h1_title}"
        else:
            filename = topic[:50]  # 如果没有h1标题，使用主题前50个字符
            
        # 清理文件名（保留问号）
        safe_filename = "".join([c if c.isalnum() or c in ('?', '？') else '_' for c in filename])
        safe_filename = safe_filename[:100]  # 限制文件名长度
        
        filepath = os.path.join(save_dir, f"{safe_filename}.txt")
        
        # 处理文件名冲突
        counter = 1
        while os.path.exists(filepath):
            filepath = os.path.join(save_dir, f"{safe_filename}_{counter}.txt")
            counter += 1
        
        with open(filepath, 'w', encoding='utf-8') as f:
            f.write(content)
        return True, filepath
    except Exception as e:
        print(f"保存文件失败: {str(e)}")
        return False, None

def get_multiline_input(prompt):
    """获取多行输入，直到用户输入空行"""
    print(prompt)
    print("(输入空行结束输入)")
    lines = []
    while True:
        line = input()
        if line == "":
            break
        lines.append(line.strip())
    return lines

def main():
    # 配置参数
    API_KEY = ""  #换成自己的API_key
    SAVE_DIR = os.path.expanduser("~/Desktop/work/txt_2")  # 保存文档目录
    
    # 获取用户输入
    print("DeepSeek批量文章生成工具（百度配图）")
    print("输入关键词（每行一个）：")
    topics = get_multiline_input("")
    
    if not topics:
        print("无有效关键词")
        return
    
    # 先获取所有主题的图片
    print("\n正在获取相关图片...")
    topic_images = {}
    for topic in topics:
        print(f"正在获取'{topic}'的图片...")
        image_url = fetch_baidu_image_url(topic)
        topic_images[topic] = image_url
        if image_url:
            print(f"获取成功")
        else:
            print("获取失败")
        time.sleep(1)  # 防止请求过于频繁
    
    # 处理每个主题
    for index, topic in enumerate(topics, 1):
        print(f"\n正在生成第 {index}/{len(topics)} 篇文章：{topic}")
        
        # 构造提示词
        prompt = f"""请撰写一篇关于{topic}的文章。要求：
        - 包含一个h1标题，格式为"核心词 + 解决方案/客户痛点"
        - 包含至少3个小节
        - 每个章节用疑问句小标题（不要用数字序号）
        - 内容主体为内容核心 + 解决方案/案例分析/客户痛点
        - 逻辑清晰,通篇大白话
        - 段落全部用<p>标签,主标题用<h1>标签，章节小标题用<h3>标签
        - 不要html框架代码和其他符号，文件保存编码为utf-8"""
        
        # 调用API生成文章
        start_time = time.time()
        content = generate_article(API_KEY, prompt)
        
        if content:
            # 插入图片
            if topic in topic_images and topic_images[topic]:
                content = insert_image_to_article(content, topic_images[topic], topic)
            
            # 保存文件
            success, filepath = save_to_txt(content, topic, SAVE_DIR)
            if success:
                elapsed = time.time() - start_time
                print(f"成功保存到：{filepath}")
                print(f"耗时：{elapsed:.2f}秒")
        else:
            print("文章生成失败")
        
        # API限速保护
        time.sleep(1)

if __name__ == "__main__":
    main()